/**
 * axios-middleware v0.1.3
 * (c) 2018 Émile Bergeron
 * @license MIT
 */
/* eslint-disable class-methods-use-this, no-unused-vars */

/**
 * Default middleware implementation. Extend this class to create a custom middleware.
 */
var HttpMiddleware = function HttpMiddleware () {};

HttpMiddleware.prototype.onRequest = function onRequest (config) {
    return config;
};

/**
 * Default implementation is a noop.
 * @param error
 */
HttpMiddleware.prototype.onRequestError = function onRequestError (error) { };

/**
 * Default implementation is the identity function.
 * @param {Object} response
 * @returns {Object}
 */
HttpMiddleware.prototype.onResponse = function onResponse (response) {
    return response;
};

/**
 * Default implementation is a noop.
 * @param error
 */
HttpMiddleware.prototype.onResponseError = function onResponseError (error) { };

var HttpMiddlewareService = function HttpMiddlewareService(axios) {
    this.middlewares = [];

    this.setHttp(axios);
};

/**
 * @param {Axios} axios
 */
HttpMiddlewareService.prototype.setHttp = function setHttp (axios) {
        var this$1 = this;

    this.unsetHttp();

    if (axios) {
        this.http = axios;

        var interceptors = axios.interceptors;

        this._requestInterceptor = interceptors.request.use(
            function (config) { return this$1._onRequest(config); },
            function (error) { return this$1._onRequestError(error); }
        );
        this._responseInterceptor = interceptors.response.use(
            function (response) { return this$1._onResponse(response); },
            function (error) { return this$1._onResponseError(error); }
        );
    }
    return this;
};

HttpMiddlewareService.prototype.unsetHttp = function unsetHttp () {
    if (this.http) {
        var interceptors = this.http.interceptors;
        interceptors.request.eject(this._requestInterceptor);
        interceptors.response.eject(this._responseInterceptor);
    }
    return this;
};

/**
 * @param {HttpMiddleware} middleware
 * @returns {boolean} true if the middleware is already registered.
 */
HttpMiddlewareService.prototype.has = function has (middleware) {
    return this.middlewares.indexOf(middleware) > -1;
};

/**
 * Add a middleware or an array of middlewares to the stack.
 * @param {HttpMiddleware|Array} middlewares
 */
HttpMiddlewareService.prototype.register = function register (middlewares) {
        var this$1 = this;

    // eslint-disable-next-line no-param-reassign
    if (!Array.isArray(middlewares)) { middlewares = [middlewares]; }

    // Test if middlewares are registered more than once.
    middlewares.forEach(function (middleware) {
        if (this$1.has(middleware)) {
            throw new Error('Middleware already registered');
        }
        this$1.middlewares.push(middleware);
    });
};

/**
 * Remove a middleware from the registered stack.
 * @param {HttpMiddleware} middleware
 */
HttpMiddlewareService.prototype.unregister = function unregister (middleware) {
    var index = this.middlewares.indexOf(middleware);
    if (index > -1) {
        this.middlewares.splice(index, 1);
    }
};

HttpMiddlewareService.prototype.reset = function reset () {
    this.middlewares.length = 0;
};

HttpMiddlewareService.prototype._onRequest = function _onRequest (config) {
    return this.middlewares.reduce(
        function (acc, middleware) { return (middleware.onRequest ? middleware.onRequest(acc) : acc); },
        config);
};

HttpMiddlewareService.prototype._onRequestError = function _onRequestError (error) {
    this.middlewares.forEach(function (middleware) { return middleware.onRequestError &&
        middleware.onRequestError(error); });
    return Promise.reject(error);
};

HttpMiddlewareService.prototype._onResponse = function _onResponse (response) {
    return this.middlewares.reduceRight(
        function (acc, middleware) { return (middleware.onResponse ? middleware.onResponse(acc) : acc); },
        response);
};

HttpMiddlewareService.prototype._onResponseError = function _onResponseError (error) {
        var this$1 = this;

    for (var i = this.middlewares.length; i--;) {
        var middleware = this$1.middlewares[i];
        if (middleware.onResponseError) { middleware.onResponseError(error); }
    }
    return Promise.reject(error);
};

var index_esm = {
    HttpMiddleware: HttpMiddleware,
    HttpMiddlewareService: HttpMiddlewareService,
    version: '0.1.3',
};

export { HttpMiddleware, HttpMiddlewareService };
export default index_esm;
